import {select,selectAll,scaleOrdinal} from 'd3';

// defining all the variables
var svg = select('svg'),
width = +svg.attr('width'), 
height = +svg.attr('height'),
width_x = width / 2,
height_y = height / 2,
colorScale = ["#87CEFA"];

//create of svg dimensions
svg.attr("preserveAspectRatio", "xMidYMid meet")
   .attr("viewBox", [0, 0, width * 2, height * 2]);

// define all nodes and markers
var defs = svg.append("defs");
		defs.selectAll("marker")
			.data(["node"])
			.enter()
			.append("marker")
				.attr("id", d => d)
				.attr("viewBox", "0 -2.5 5 5")
				.attr("refX", 16)
				.attr("refY", 0)
				.attr("markerWidth", 5)
				.attr("markerHeight", 5)
				.attr("orient", "auto")
			.append("path")
				.attr("d", "M0,-2.5L5,0L0,2.5")
				.attr("fill", "#999");
// the data is being loaded into the graph
d3.json('network.json')
  .then(data => {
  
  // define links
  var links = data.links.map(d => Object.create(d));
  links.forEach((d, i) => {
    d.srcType = data.links[i].srcType;
    d.tgtType = data.links[i].tgtType;
    d.relationship = data.links[i].relationship;
    d.value = data.links[i].value;
  });
  
  //define nodes
  var nodes = data.nodes.map(d => Object.create(d));
  nodes.forEach((d, i) => {
  	d.group = data.nodes[i].group;
  });

  // generate links
  var simulation = d3.forceSimulation(nodes)
      .force("link", d3.forceLink(links)
             .id(d => d.id)
             .distance(200))
      .force("charge", d3.forceManyBody()
             .strength(-500)
            )
  		.force("collision", d3.forceCollide()
             	.radius(35)
            )
      .force("center", d3.forceCenter(width, height))
  ;

  //define events
  var drag = simulation => {
    function dragstarted(event) {
      if (!event.active) simulation.alphaTarget(0.5).restart();
      event.subject.fx = event.subject.x;
      event.subject.fy = event.subject.y;
    }
    function dragged(event) {
      event.subject.fx = event.x;
      event.subject.fy = event.y;
    }
    function dragended(event, d) {
      if (!event.active) simulation.alphaTarget(0);
      event.subject.fx = null;
      event.subject.fy = null;
    }
    return d3.drag()
      .on("start", dragstarted)
      .on("drag", dragged)
      .on("end", dragended);
  }

  var link = svg.append("g")
  .selectAll("line")
    .data(links)
  	.join("line")
  		.attr("marker-end", "url(#node)");
    
  var node = svg.append("g")
    .attr("class", "node")
  .selectAll("g")
    .data(nodes)
  	.enter()
  	.append("g")
    	.attr("transform", d => "translate(" + width_x + ", " + height_y + ")");
;
  //creating the cirles and shaping them.
  var circles = node.append("circle")
  		.attr("r", 70)
  		.attr("fill", d => colorScale[0])
			.on("mouseover", focusNode)
			.on("mouseout", blurNode)
  		.call(drag(simulation));

  
  // tagging a node with text
  var nodeLbl = node.append("text")
  			.text(d => d.id)
  			.attr("dy", "0.7em");
  // highlight the nodes using pointers
 	simulation.on("tick", () => {
    link
 			.attr("x1", d => d.source.x)                            
 			.attr("y1", d => d.source.y)
 			.attr("x2", d => d.target.x)
 			.attr("y2", d => d.target.y);
    node.attr("transform", d => "translate(" + d.x + ", " + d.y + ")");
  });
  
	function focusNode() {
	  d3.select(this).classed("nodeHover", true);
	}

	function blurNode() {
		d3.select(this).classed("nodeHover", false);
	}

});
